;;; oni-circe.el --- Circe configuration             -*- lexical-binding: t; -*-

;; Copyright (C) 2019  Tom Willemse

;; Author: Tom Willemse <tom@ryuslash.org>
;; Keywords: local
;; Version: 20190226004925
;; Package-Requires: (circe oni-lui)

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>.

;;; Commentary:

;; Configuration for the Circe IRC client.

;;; Code:

(require 'circe)
(require 'circe-color-nicks)
(require 'circe-serenity)

(defun oni-circe-get-password-for (host)
  "Create a function that will get a password for HOST.
The returned function will look for the password in .authinfo,
which may be encrypted."
  (lambda (_)
    (let ((found (nth 0 (auth-source-search :max 1
                                            :host host
                                            :require '(:secret)))))
      (when found
        (let ((secret (plist-get found :secret)))
          (if (functionp secret)
              (funcall secret)
            secret))))))

(add-to-list 'circe-network-options
             `("Freenode"
               :nick "ryuslash"
               :channels ("#mowedline"
                          "#ninthfloor"
                          "#dispass"
                          "#linuxvoice"
                          "#conkeror"
                          "#emacs-circe"
                          "#chicken"
                          :after-auth
                          "#emacs")
               :nickserv-password
               ,(oni-circe-get-password-for "irc.freenode.net")))

(add-to-list 'circe-network-options
             `("Twitch"
               :use-tls nil
               :nick "ryuslash"
               :host "irc.twitch.tv"
               :pass ,(oni-circe-get-password-for "irc.twitch.tv")
               :port 6667))

(enable-circe-color-nicks)
(enable-circe-serenity)

;; (require 'mowedline)

;; (defun mowedline-colorize-mode-line-spec (s)
;;   (cond
;;    ((stringp s) (string-trim s))
;;    ((eq ':propertize (car s))
;;     (let ((s (cadr s))
;;           (face (plist-get (cddr s) 'face)))
;;       (if face
;;           `(color ,(face-foreground face)
;;                   ,(substring-no-properties s))
;;         s)))
;;    (t "")))

;; (setq mowedline-update-function 'mowedline-update/dbus)

;; (defvar jjf-tracking-buffers '())
;; (defvar jjf-external-tracking-timer nil)
;; (defun jjf-external-tracking ()
;;   (setq jjf-external-tracking-timer nil)
;;   (mowedline-update
;;    'irc
;;    (if (stringp jjf-tracking-buffers)
;;        (mowedline-colorize jjf-tracking-buffers t)
;;      (format "%S" (mapcar #'mowedline-colorize-mode-line-spec
;;                           jjf-tracking-buffers)))))

;; (defun oni:clear-irc-mowedline-widget ()
;;   (mowedline-clear 'irc))

;; (defun jjf-external-tracking-advice (orig-fun &rest args)
;;   "Update my external status bar when tracking computes a new
;; status line, and suppress tracking in the mode-line.  Since
;; tracking-status may be called many times in quick succession, for
;; example on a make-frame-visible event, we use a short timer to
;; only call the updater once within a minimum duration."
;;   (setq jjf-tracking-buffers (apply orig-fun args))
;;   (when jjf-external-tracking-timer
;;     (cancel-timer jjf-external-tracking-timer))
;;   (setq jjf-external-tracking-timer
;;         (run-at-time 0.2 nil 'jjf-external-tracking))
;;   nil)

;;(advice-add 'tracking-status :around #'jjf-external-tracking-advice)
;;(add-hook 'kill-emacs-hook 'oni:clear-irc-mowedline-widget)

;;;###autoload(with-eval-after-load 'circe (require 'oni-circe))

(provide 'oni-circe)
;;; oni-circe.el ends here
